@@ -2,7 +2,7 @@ package Net::DNS::RR::DNSKEY;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: DNSKEY.pm 1972 2024-04-21 08:13:19Z willem $)[2];
+our $VERSION = (qw$Id: DNSKEY.pm 2042 2025-12-24 10:23:11Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -83,9 +83,8 @@ sub flags {
 
 sub zone {
 	my ( $self, @value ) = @_;
-	for ( $self->{flags} |= 0 ) {
-		if ( scalar @value ) {
-			$_ |= 0x0100;
+	if ( scalar @value ) {
+		for ( $self->{flags} |= 0x0100 ) {
 			$_ ^= 0x0100 unless shift @value;
 		}
 	}
@@ -95,9 +94,8 @@ sub zone {
 
 sub revoke {
 	my ( $self, @value ) = @_;
-	for ( $self->{flags} |= 0 ) {
-		if ( scalar @value ) {
-			$_ |= 0x0080;
+	if ( scalar @value ) {
+		for ( $self->{flags} |= 0x0080 ) {
 			$_ ^= 0x0080 unless shift @value;
 		}
 	}
@@ -105,11 +103,21 @@ sub revoke {
 }
 
 
+sub adt {
+	my ( $self, @value ) = @_;
+	if ( scalar @value ) {
+		for ( $self->{flags} |= 0x0002 ) {
+			$_ ^= 0x0002 unless shift @value;
+		}
+	}
+	return $self->{flags} & 0x0002;
+}
+
+
 sub sep {
 	my ( $self, @value ) = @_;
-	for ( $self->{flags} |= 0 ) {
-		if ( scalar @value ) {
-			$_ |= 0x0001;
+	if ( scalar @value ) {
+		for ( $self->{flags} |= 0x0001 ) {
 			$_ ^= 0x0001 unless shift @value;
 		}
 	}
@@ -240,8 +248,8 @@ sub keytag {
 		'ECDSAP384SHA384'    => 14,			# [RFC6605]
 		'ED25519'	     => 15,			# [RFC8080]
 		'ED448'		     => 16,			# [RFC8080]
-		'SM2SM3'	     => 17,			# [RFC-cuiling-dnsop-sm2-alg-15]
-		'ECC-GOST12'	     => 23,			# [RFC-makarenko-gost2012-dnssec-05]
+		'SM2SM3'	     => 17,			# [RFC9563]
+		'ECC-GOST12'	     => 23,			# [RFC9558]
 
 		'INDIRECT'   => 252,				# [RFC4034]
 		'PRIVATEDNS' => 253,				# [RFC4034]
@@ -279,8 +287,8 @@ __END__
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
-    $rr = Net::DNS::RR->new('name DNSKEY flags protocol algorithm publickey');
+	use Net::DNS;
+	$rr = Net::DNS::RR->new('name DNSKEY flags protocol algorithm publickey');
 
 =head1 DESCRIPTION
 
@@ -298,8 +306,8 @@ other unpredictable behaviour.
 
 =head2 flags
 
-    $flags = $rr->flags;
-    $rr->flags( $flags );
+	$flags = $rr->flags;
+	$rr->flags( $flags );
 
 Unsigned 16-bit number representing Boolean flags.
 
@@ -307,11 +315,11 @@ Unsigned 16-bit number representing Boolean flags.
 
 =item zone
 
- $rr->zone(1);
+	$rr->zone(1);
 
- if ( $rr->zone ) {
-	...
- }
+	if ( $rr->zone ) {
+		...
+	}
 
 Boolean ZONE flag.
 
@@ -321,11 +329,11 @@ Boolean ZONE flag.
 
 =item revoke
 
- $rr->revoke(1);
+	$rr->revoke(1);
 
- if ( $rr->revoke ) {
-	...
- }
+	if ( $rr->revoke ) {
+		...
+	}
 
 Boolean REVOKE flag.
 
@@ -333,13 +341,27 @@ Boolean REVOKE flag.
 
 =over 4
 
+=item adt
+
+	$rr->adt(1);
+
+	if ( $rr->adt ) {
+		...
+	}
+
+Authoritative Delegation Types (ADT) flag.
+
+=back
+
+=over 4
+
 =item sep
 
- $rr->sep(1);
+	$rr->sep(1);
 
- if ( $rr->sep ) {
-	...
- }
+	if ( $rr->sep ) {
+		...
+	}
 
 Boolean Secure Entry Point (SEP) flag.
 
@@ -347,15 +369,15 @@ Boolean Secure Entry Point (SEP) flag.
 
 =head2 protocol
 
-    $protocol = $rr->protocol;
-    $rr->protocol( $protocol );
+	$protocol = $rr->protocol;
+	$rr->protocol( $protocol );
 
 The 8-bit protocol number.  This field MUST have value 3.
 
 =head2 algorithm
 
-    $algorithm = $rr->algorithm;
-    $rr->algorithm( $algorithm );
+	$algorithm = $rr->algorithm;
+	$rr->algorithm( $algorithm );
 
 The 8-bit algorithm number describes the public key algorithm.
 
@@ -366,21 +388,21 @@ to perform mnemonic and numeric code translation.
 
 =head2 key
 
-    $key = $rr->key;
-    $rr->key( $key );
+	$key = $rr->key;
+	$rr->key( $key );
 
 Base64 representation of the public key material.
 
 =head2 keybin
 
-    $keybin = $rr->keybin;
-    $rr->keybin( $keybin );
+	$keybin = $rr->keybin;
+	$rr->keybin( $keybin );
 
 Opaque octet string representing the public key material.
 
 =head2 privatekeyname
 
-    $privatekeyname = $rr->privatekeyname;
+	$privatekeyname = $rr->privatekeyname;
 
 Returns the name of the privatekey as it would be generated by
 the BIND dnssec-keygen program. The format of that name being:
@@ -389,6 +411,8 @@ the BIND dnssec-keygen program. The format of that name being:
 
 =head2 signame
 
+	$signame = $rr->signame;
+
 Returns the canonical signer name of the privatekey.
 
 =head2 keylength
@@ -397,7 +421,7 @@ Returns the length (in bits) of the modulus calculated from the key text.
 
 =head2 keytag
 
-    print "keytag = ", $rr->keytag, "\n";
+	print "keytag = ", $rr->keytag, "\n";
 
 Returns the 16-bit numerical key tag of the key. (RFC2535 4.1.6)
 
