@@ -2,7 +2,7 @@ package Net::DNS::RR::DS;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: DS.pm 1957 2024-01-10 14:54:10Z willem $)[2];
+our $VERSION = (qw$Id: DS.pm 2042 2025-12-24 10:23:11Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -22,9 +22,10 @@ use constant BABBLE => defined eval { require Digest::BubbleBabble };
 eval { require Digest::SHA };		## optional for simple Net::DNS RR
 
 my %digest = (
-	'1' => scalar( eval { Digest::SHA->new(1) } ),
-	'2' => scalar( eval { Digest::SHA->new(256) } ),
-	'4' => scalar( eval { Digest::SHA->new(384) } ),
+	'1' => ['Digest::SHA', 1],
+	'2' => ['Digest::SHA', 256],
+	'4' => ['Digest::SHA', 384],
+	'6' => ['Net::DNS::SEC::Digest::SM3'],
 	);
 
 
@@ -144,12 +145,15 @@ sub create {
 		keytag	  => $keyrr->keytag
 		);
 
-	my $hash = $digest{$self->digtype};
+	my $spec = $digest{$self->digtype};
+	my $hash = eval {
+		my ( $object, @param ) = @$spec;
+		$object->new(@param);
+	};
 	croak join ' ', 'digtype', $self->digtype('MNEMONIC'), 'not supported' unless $hash;
-	my $clone = $hash->clone;
-	$clone->add( $keyrr->{owner}->canonical );
-	$clone->add( $keyrr->_encode_rdata );
-	$self->digestbin( $clone->digest );
+	$hash->add( $keyrr->{owner}->canonical );
+	$hash->add( $keyrr->_encode_rdata );
+	$self->digestbin( $hash->digest );
 
 	return $self;
 }
@@ -166,15 +170,15 @@ sub verify {
 
 {
 	my @digestbyname = (
-		'SHA-1'		  => 1,				# [RFC3658]
-		'SHA-256'	  => 2,				# [RFC4509]
-		'GOST-R-34.11-94' => 3,				# [RFC5933]
-		'SHA-384'	  => 4,				# [RFC6605]
+		'SHA-1'		    => 1,			# [RFC3658]
+		'SHA-256'	    => 2,			# [RFC4509]
+		'GOST-R-34.11-94'   => 3,			# [RFC5933]
+		'SHA-384'	    => 4,			# [RFC6605]
+		'GOST-R-34.11-2012' => 5,			# [RFC9558]
+		'SM3'		    => 6,			# [RFC9563]
 		);
 
-	my @digestalias = (
-		'SHA'	 => 1,
-		);
+	my @digestalias = ( 'SHA' => 1 );
 
 	my %digestbyval = reverse @digestbyname;
 
@@ -217,6 +221,8 @@ sub verify {
 		'ECDSAP384SHA384'    => 14,			# [RFC6605]
 		'ED25519'	     => 15,			# [RFC8080]
 		'ED448'		     => 16,			# [RFC8080]
+		'SM2SM3'	     => 17,			# [RFC9563]
+		'ECC-GOST12'	     => 23,			# [RFC9558]
 
 		'INDIRECT'   => 252,				# [RFC4034]
 		'PRIVATEDNS' => 253,				# [RFC4034]
@@ -254,15 +260,15 @@ __END__
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
-    $rr = Net::DNS::RR->new('name DS keytag algorithm digtype digest');
+	use Net::DNS;
+	$rr = Net::DNS::RR->new('name DS keytag algorithm digtype digest');
 
-    use Net::DNS::SEC;
-    $ds = Net::DNS::RR::DS->create(
-	$dnskeyrr,
-	digtype => 'SHA256',
-	ttl	=> 3600
-	);
+	use Net::DNS::SEC;
+	$ds = Net::DNS::RR::DS->create(
+			$dnskeyrr,
+			digtype => 'SHA256',
+			ttl	=> 3600
+			);
 
 =head1 DESCRIPTION
 
@@ -280,15 +286,15 @@ other unpredictable behaviour.
 
 =head2 keytag
 
-    $keytag = $rr->keytag;
-    $rr->keytag( $keytag );
+	$keytag = $rr->keytag;
+	$rr->keytag( $keytag );
 
 The 16-bit numerical key tag of the key. (RFC2535 4.1.6)
 
 =head2 algorithm
 
-    $algorithm = $rr->algorithm;
-    $rr->algorithm( $algorithm );
+	$algorithm = $rr->algorithm;
+	$rr->algorithm( $algorithm );
 
 Decimal representation of the 8-bit algorithm field.
 
@@ -297,8 +303,8 @@ to perform mnemonic and numeric code translation.
 
 =head2 digtype
 
-    $digtype = $rr->digtype;
-    $rr->digtype( $digtype );
+	$digtype = $rr->digtype;
+	$rr->digtype( $digtype );
 
 Decimal representation of the 8-bit digest type field.
 
@@ -307,21 +313,21 @@ to perform mnemonic and numeric code translation.
 
 =head2 digest
 
-    $digest = $rr->digest;
-    $rr->digest( $digest );
+	$digest = $rr->digest;
+	$rr->digest( $digest );
 
 Hexadecimal representation of the digest over the label and key.
 
 =head2 digestbin
 
-    $digestbin = $rr->digestbin;
-    $rr->digestbin( $digestbin );
+	$digestbin = $rr->digestbin;
+	$rr->digestbin( $digestbin );
 
 Binary representation of the digest over the label and key.
 
 =head2 babble
 
-    print $rr->babble;
+	print $rr->babble;
 
 The babble() method returns the 'BubbleBabble' representation of the
 digest if the Digest::BubbleBabble package is available, otherwise
@@ -337,11 +343,11 @@ method is called.
 
 =head2 create
 
-    use Net::DNS::SEC;
+	use Net::DNS::SEC;
 
-    $dsrr = Net::DNS::RR::DS->create( $keyrr, digtype => 'SHA-256' );
-    $keyrr->print;
-    $dsrr->print;
+	$dsrr = Net::DNS::RR::DS->create( $keyrr, digtype => 'SHA-256' );
+	$keyrr->print;
+	$dsrr->print;
 
 This constructor takes a DNSKEY argument and will return the
 corresponding DS RR constructed using the specified algorithm.
@@ -350,7 +356,7 @@ The digest algorithm defaults to SHA-1.
 
 =head2 verify
 
-    $verify = $dsrr->verify($keyrr);
+	$verify = $dsrr->verify($keyrr);
 
 The boolean verify method will return true if the hash over the key
 RR provided as the argument conforms to the data in the DS itself
@@ -390,10 +396,10 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<perl> L<Net::DNS> L<Net::DNS::RR>
-L<RFC4034|https://tools.ietf.org/html/rfc4034>
+L<RFC4034(5)|https://iana.org/go/rfc4034#section-5>
 
-L<Digest Types|http://www.iana.org/assignments/ds-rr-types>
+L<Digest Types|https://iana.org/assignments/ds-rr-types>
 
-L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
+L<Algorithm Numbers|https://iana.org/assignments/dns-sec-alg-numbers>
 
 =cut
