@@ -2,7 +2,7 @@ package Net::DNS::RR::SIG;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: SIG.pm 1990 2024-09-18 13:16:07Z willem $)[2];
+our $VERSION = (qw$Id: SIG.pm 2042 2025-12-24 10:23:11Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -29,17 +29,19 @@ eval { require MIME::Base64 };
 ## IMPORTANT: MUST NOT include crypto packages in metadata (strong crypto prohibited in many territories)
 use constant DNSSEC => defined $INC{'Net/DNS/SEC.pm'};	## Discover how we got here, without exposing any crypto
 
-my @index;
+my @algorithms;
+my @deprecated;
 if (DNSSEC) {
-	foreach my $class ( map {"Net::DNS::SEC::$_"} qw(Private RSA DSA ECDSA EdDSA Digest SM2) ) {
-		my @algorithms = eval join '', qw(r e q u i r e), " $class; ${class}::_index()";	## no critic
-		push @index, map { ( $_ => $class ) } @algorithms;
+	foreach my $class ( map {"Net::DNS::SEC::$_"} qw(Private DSA RSA ECDSA EdDSA Digest SM2) ) {
+		my @index = eval join '', qw(r e q u i r e), " $class; ${class}::_index()";	## no critic
+		push @algorithms, map { ( $_ => $class ) } @index;
+		push @deprecated, eval "${class}::_deprecate()";			   	## no critic
 	}
-	croak 'Net::DNS::SEC version not supported' unless scalar(@index);
+	croak 'Net::DNS::SEC version not supported' unless scalar(@algorithms);
 }
 
-my %DNSSEC_verify = @index;
-my %DNSSEC_siggen = @index;
+my %DNSSEC_verify = @algorithms;
+my %DNSSEC_siggen = @algorithms;
 
 my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 
@@ -349,8 +351,8 @@ sub vrfyerrstr {
 		'ECDSAP384SHA384'    => 14,			# [RFC6605]
 		'ED25519'	     => 15,			# [RFC8080]
 		'ED448'		     => 16,			# [RFC8080]
-		'SM2SM3'	     => 17,			# [RFC-cuiling-dnsop-sm2-alg-15]
-		'ECC-GOST12'	     => 23,			# [RFC-makarenko-gost2012-dnssec-05]
+		'SM2SM3'	     => 17,			# [RFC9563]
+		'ECC-GOST12'	     => 23,			# [RFC9558]
 
 		'INDIRECT'   => 252,				# [RFC4034]
 		'PRIVATEDNS' => 253,				# [RFC4034]
@@ -541,18 +543,19 @@ __END__
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
-    $rr = Net::DNS::RR->new('name SIG typecovered algorithm labels
+	use Net::DNS;
+	$rr = Net::DNS::RR->new('name SIG typecovered algorithm labels
 				orgttl sigexpiration siginception
 				keytag signame signature');
 
-    use Net::DNS::SEC;
-    $sigrr = Net::DNS::RR::SIG->create( $string, $keypath,
-					sigval => 10	# minutes
-					);
+	use Net::DNS::SEC;
+	$sigrr = Net::DNS::RR::SIG->create(
+				$string, $keypath,
+				sigval => 10	# minutes
+				);
 
-    $sigrr->verify( $string, $keyrr ) || die $sigrr->vrfyerrstr;
-    $sigrr->verify( $packet, $keyrr ) || die $sigrr->vrfyerrstr;
+	$sigrr->verify( $string, $keyrr ) || die $sigrr->vrfyerrstr;
+	$sigrr->verify( $packet, $keyrr ) || die $sigrr->vrfyerrstr;
 
 =head1 DESCRIPTION
 
@@ -577,7 +580,7 @@ other unpredictable behaviour.
 
 =head2 algorithm
 
-    $algorithm = $rr->algorithm;
+	$algorithm = $rr->algorithm;
 
 The algorithm number field identifies the cryptographic algorithm
 used to create the signature.
@@ -589,11 +592,11 @@ to perform mnemonic and numeric code translation.
 
 =head2 sigex sigin sigval
 
-    $expiration = $rr->sigexpiration;
-    $expiration = $rr->sigexpiration( $value );
+	$expiration = $rr->sigexpiration;
+	$expiration = $rr->sigexpiration( $value );
 
-    $inception = $rr->siginception;
-    $inception = $rr->siginception( $value );
+	$inception = $rr->siginception;
+	$inception = $rr->siginception( $value );
 
 The signature expiration and inception fields specify a validity
 time interval for the signature.
@@ -606,16 +609,16 @@ numerical Perl time() value.
 
 =head2 keytag
 
-    $keytag = $rr->keytag;
-    $rr->keytag( $keytag );
+	$keytag = $rr->keytag;
+	$rr->keytag( $keytag );
 
 The keytag field contains the key tag value of the KEY RR that
 validates this signature.
 
 =head2 signame
 
-    $signame = $rr->signame;
-    $rr->signame( $signame );
+	$signame = $rr->signame;
+	$rr->signame( $signame );
 
 The signer name field value identifies the owner name of the KEY
 RR that a validator is supposed to use to validate this signature.
@@ -624,16 +627,16 @@ RR that a validator is supposed to use to validate this signature.
 
 =head2 sig
 
-    $sig = $rr->sig;
-    $rr->sig( $sig );
+	$sig = $rr->sig;
+	$rr->sig( $sig );
 
 The Signature field contains the cryptographic signature that covers
 the SIG RDATA (excluding the Signature field) and the subject data.
 
 =head2 sigbin
 
-    $sigbin = $rr->sigbin;
-    $rr->sigbin( $sigbin );
+	$sigbin = $rr->sigbin;
+	$rr->sigbin( $sigbin );
 
 Binary representation of the cryptographic signature.
 
@@ -641,23 +644,24 @@ Binary representation of the cryptographic signature.
 
 Create a signature over scalar data.
 
-    use Net::DNS::SEC;
+	use Net::DNS::SEC;
 
-    $keypath = '/home/olaf/keys/Kbla.foo.+001+60114.private';
+	$keypath = '/home/olaf/keys/Kbla.foo.+001+60114.private';
 
-    $sigrr = Net::DNS::RR::SIG->create( $data, $keypath );
+	$sigrr = Net::DNS::RR::SIG->create( $data, $keypath );
 
-    $sigrr = Net::DNS::RR::SIG->create( $data, $keypath,
-					sigval => 10
-					);
-    $sigrr->print;
+	$sigrr = Net::DNS::RR::SIG->create(
+				$data, $keypath,
+				sigval => 10
+				);
+	$sigrr->print;
 
 
-    # Alternatively use Net::DNS::SEC::Private 
+	# Alternatively use Net::DNS::SEC::Private 
 
-    $private = Net::DNS::SEC::Private->new($keypath);
+	$private = Net::DNS::SEC::Private->new($keypath);
 
-    $sigrr= Net::DNS::RR::SIG->create( $data, $private );
+	$sigrr= Net::DNS::RR::SIG->create( $data, $private );
 
 
 create() is an alternative constructor for a SIG RR object.  
@@ -674,8 +678,8 @@ that comes with the ISC BIND distribution.
 The optional remaining arguments consist of ( name => value ) pairs
 as follows:
 
-	sigin  => 20241201010101,	# signature inception
-	sigex  => 20241201011101,	# signature expiration
+	sigin  => 20251201010101,	# signature inception
+	sigex  => 20251201011101,	# signature expiration
 	sigval => 10,			# validity window (minutes)
 
 The sigin and sigex values may be specified as Perl time values or as
@@ -687,20 +691,10 @@ The sigval argument specifies the signature validity window in minutes
 
 By default the signature is valid for 10 minutes.
 
-=over 4
-
-=item *
-
-Do not change the name of the private key file.
-The create method uses the filename as generated by dnssec-keygen
-to determine the keyowner, algorithm, and the keyid (keytag).
-
-=back
-
 =head2 verify
 
-    $verify = $sigrr->verify( $data, $keyrr );
-    $verify = $sigrr->verify( $data, [$keyrr, $keyrr2, $keyrr3] );
+	$verify = $sigrr->verify( $data, $keyrr );
+	$verify = $sigrr->verify( $data, [$keyrr, $keyrr2, $keyrr3] );
 
 The verify() method performs SIG0 verification of the specified data
 against the signature contained in the $sigrr object itself using
@@ -718,10 +712,26 @@ Returns false on error and sets $sig->vrfyerrstr
 
 =head2 vrfyerrstr
 
-    $sig0 = $packet->sigrr || die 'not signed';
-    print $sig0->vrfyerrstr unless $sig0->verify( $packet, $keyrr );
+	$sig0 = $packet->sigrr || die 'not signed';
+	print $sig0->vrfyerrstr unless $sig0->verify( $packet, $keyrr );
+
+	$sigrr->verify( $packet, $keyrr ) || die $sigrr->vrfyerrstr;
+
+=head1 KEY GENERATION
+
+Private key files and corresponding public DNSKEY records
+are most conveniently generated using dnssec-keygen,
+a program that comes with the ISC BIND distribution.
+
+	dnssec-keygen -a 10 -b 2048 rsa.example.
+
+	dnssec-keygen -a 13 -f ksk  ecdsa.example.
+	dnssec-keygen -a 13	    ecdsa.example.
+
+Do not change the name of the private key file.
+The create method uses the filename as generated by dnssec-keygen
+to determine the keyowner, algorithm, and the keyid (keytag).
 
-    $sigrr->verify( $packet, $keyrr ) || die $sigrr->vrfyerrstr;
 
 =head1 REMARKS
 
@@ -741,6 +751,8 @@ Andy Vaskys (Network Associates Laboratories) supplied code for RSA.
 
 T.J. Mather provided support for the DSA algorithm.
 
+Dick Franks added support for elliptic curve and Edwards curve algorithms.
+
 
 =head1 COPYRIGHT
 
